/*:
 * @plugindesc (v1.1) Adjust AnotherNewGame Position
 *
 * @param Another New Game X-Axis
 * @desc x
 * @default 0
 *
 * @param Another New Game Y-Axis
 * @desc y
 * @default 0
 *
 */


(function(){
    var TitlePictureCom_parameters = PluginManager.parameters('MOG_TitlePictureCom');
    var parameters = PluginManager.parameters('liply_TitlePictureCom_AnotherNewGame_Bridge');

    function getValue(obj, name, defaultValue){
        if(obj[name] === undefined){
            return defaultValue;
        }

        return +obj[name];
    }

    function myParameter(name, defaultValue){
        return getValue(parameters, name, defaultValue);
    }

    function TPCParameter(name, defaultValue){
        return getValue(TitlePictureCom_parameters, name, defaultValue);
    }

    var anotherNewGamePosition = [
        myParameter('Another New Game X-Axis', 0),
        myParameter('Another New Game Y-Axis', 0)
    ];

    var newGamePosition = [
        TPCParameter('New Game X-Axis', 0),
        TPCParameter('New Game Y-Axis', 0)
    ];

    var continuePosition = [
        TPCParameter('Continue X-Axis', 0),
        TPCParameter('Continue Y-Axis', 32)
    ];

    var optionsPosition = [
        TPCParameter('Options X-Axis', 0),
        TPCParameter('Options Y-Axis', 64)
    ];

    function fetchPosition(symbol){
        switch(symbol){
            case 'nameGame2':
                return anotherNewGamePosition;
            case 'newGame':
                return newGamePosition;
            case 'continue':
                return continuePosition;
            case 'options':
                return optionsPosition;
        }

        return [0,0];
    }

    function getTargetSpriteIndex(symbol){
        switch(symbol){
            case 'nameGame2':
                return 3;
            case 'newGame':
                return 0;
            case 'continue':
                return 1;
            case 'options':
                return 2;
        }

        return 0;
    }

    function adjustSpritePosition(sprite, picture, basePosition, selected){
        var ch = selected? picture.height/2: 0;

        sprite.setFrame(0, ch, picture.width, picture.height / 2);
        sprite.x = ((Graphics.boxWidth / 2) - (picture.width / 2)) + basePosition[0];
        sprite.y = ((Graphics.boxHeight / 2) + (picture.height / 2)) + basePosition[1];
    }

    function updatePictureData(data, sprites, pictures){
        for(var i = 0; i < sprites.length; i++){
            data[i] = [sprites[i].x, pictures[i].width ,sprites[i].y, pictures[i].height / 2 ];
        }
    }

    Scene_Title.prototype.refresh_picture_command = function() {
        var commandList = this._commandWindow._list;
        for(var n = 0; n < commandList.length; n++){
            var symbol = commandList[n].symbol;
            var basePosition = fetchPosition(symbol);
            var spriteIndex = getTargetSpriteIndex(symbol);
            var sprite = this._com_sprites[spriteIndex];
            var picture = this._com_pictures[spriteIndex];

            adjustSpritePosition(sprite, picture, basePosition, n !== this._commandWindow._index);

            if (symbol === 'continue' && !this._commandWindow.isContinueEnabled()) {
                sprite.opacity = 160;
            }
        }
        updatePictureData(this._com_pictures_data, this._com_sprites, this._com_pictures);
        this._com_index_old = this._commandWindow._index;
    };
})();